﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Data;
using System.Web.UI.HtmlControls;

namespace Infra
{


public partial class _Default : System.Web.UI.Page 
{
    
        //protected System.Web.UI.WebControls.Button Button1;
		//protected Infragistics.WebUI.UltraWebGrid.UltraWebGrid UltraWebGrid1;
		private bool isFiltering = false;
	
		private void Page_Load(object sender, System.EventArgs e)
		{
            // If there is a query string, then this is a custom AJAX call, which means that the user has changed a
			// DropDownList selection from the Dropdown. 
			if (Request.QueryString.Count > 0)
			{
				// Filter the DataView
				applyFilterOnDataView();
				isFiltering = true;
			}
		}

		#region InitializeDatasource For Grid
		// This event is the key event for the grid to handle everything for the grid.  There could be three different
		// scenarios where we will make a server request, depending on what is causing the server request. We need to
		// handle this event to set the datasource for the grid accordingly. 
		       	#endregion

		#region Helper Functions
		// Sets the filter on the DataView with the parameters passed from the client, and stores these in session state
		private void applyFilterOnDataView()
		{
			string ColumnName = Request.QueryString.GetKey(0).ToString();
			string FilterValue = Request.QueryString[0].ToString();

			DataView dv = (DataView) Session["GridDataSource"];

			if(FilterValue == "(ALL)")
				dv.RowFilter = "";
			else
                dv.RowFilter = ColumnName + " like '" + FilterValue + "'";

			Session["GridDataSource"] = dv;
		}

		// Creates all templated columns for the WebGrid, and assigns the header template for the grid to use a DropDownList
		private void createDropDownsForRowFiltering(DataView dv)
		{
			foreach (DataColumn thisCol in dv.Table.Columns)
			{
				Infragistics.WebUI.UltraWebGrid.TemplatedColumn tempCol = new Infragistics.WebUI.UltraWebGrid.TemplatedColumn(true);
				tempCol.BaseColumnName = thisCol.ColumnName;
				tempCol.Key = thisCol.ColumnName;
				tempCol.Header.Caption = thisCol.ColumnName;
				tempCol.HeaderTemplate = new HeaderTemplate(dv.Table.DataSet);
				this.UltraWebGrid1.Columns.Add(tempCol);
			}
		}
		#endregion



        protected void UltraWebGrid1_InitializeDataSource1(object sender, Infragistics.WebUI.UltraWebGrid.UltraGridEventArgs e)
        {
            // See if this is a first load, or if this is a postback that is not an AJAX postback made by the WebGrid
            if (!this.IsPostBack && !this.UltraWebGrid1.IsXmlHttpRequest && !isFiltering)
            {
                // 1: Fill the DataSet for the first time
                DataSet ds = new DataSet();
                ds.ReadXml(Request.PhysicalApplicationPath + "NWOrdersData.xml");

                // 2: Store the DefaultView of the DataTable in session state
                Session["GridDataSource"] = ds.Tables[0].DefaultView;

                // 3: Create templated columns for DropDownLists to show in each header
                createDropDownsForRowFiltering(ds.Tables[0].DefaultView);

                // Since we want the DropDownLists for the header template, we will create all columns in code
                this.UltraWebGrid1.DisplayLayout.AutoGenerateColumns = false;
            }
            else if (this.IsPostBack && !this.UltraWebGrid1.IsXmlHttpRequest)
            {
                // 1: Get the DataView from the session state
                DataView dv = (DataView)Session["GridDataSource"];

                // 2: Create templated columns for DropDownLists to show in each header
                createDropDownsForRowFiltering(dv);

                // Since we want the DropDownLists for the header template, we will create all columns in code
                this.UltraWebGrid1.DisplayLayout.AutoGenerateColumns = false;
            }

            // Set the WebGrid's DataSource to what is stored in session state
            this.UltraWebGrid1.DataSource = Session["GridDataSource"];

            // Clear the isFiltering flag
            isFiltering = false;

        }
}

	#region Helper classes
	// This class is used to get distinct values from DataSet tables
	public class DataSetHelper
	{
		public DataSet ds;

		public DataSetHelper(ref DataSet dataSet)
		{
			ds = dataSet;
		}

		public DataSetHelper()
		{
			ds = null;
		}

		// Compares two values to see if they are equal. DBNull.Value is equal to DBNull.Value in this method.
		private bool ColumnEqual(object A, object B)
		{
			// Note: If your DataTable contains object fields, then you must extend this
			// function to handle them in a meaningful way if you intend to group on them
			if (A == DBNull.Value && B == DBNull.Value) // both are DBNull.Value
				return true;
			if (A == DBNull.Value || B == DBNull.Value) // only one is DBNull.Value
				return false;
			return (A.Equals(B));  // value type standard comparison
		}

		public DataTable SelectDistinct(string TableName, DataTable SourceTable, string FieldName)
		{
			DataTable dt = new DataTable(TableName);
			dt.Columns.Add(FieldName, SourceTable.Columns[FieldName].DataType);

			dt.Rows.Add(new object[] { "(ALL)" });
			object LastValue = null;

			foreach (DataRow dr in SourceTable.Select("", FieldName))
			{
				if (LastValue == null || !(ColumnEqual(LastValue, dr[FieldName])) )
				{
					LastValue = dr[FieldName];
					dt.Rows.Add(new object[]{LastValue});
				}
			}

			if (ds != null)
				ds.Tables.Add(dt);

			return dt;
		}
	}

	// This class is used to create filter DropDowns in the column header templates
	public class HeaderTemplate : System.Web.UI.ITemplate
	{
		private DataSet gridData = new DataSet();
		private DataSetHelper ddldsHelper;

		// Use the contructor that will pass the DataSet object to used to created the DropDownLists for header templates
		public HeaderTemplate(DataSet ds)
		{
			gridData = ds;
			ddldsHelper = new DataSetHelper(ref gridData);
		}
		
		#region ITemplate Members
		// Use the InstantiateIn method to create the DropDownList and add it to the column's header area.
		// As this will be called for each of the header columns, we will use the column's Key to match the DataTable
		// columns to reference them easily. 
		// This method is called every time a column header will be created for the grid, so we can use the same code
		// to set up a DropDownList.
		public void InstantiateIn(Control container)
		{
			// Cast the container to a HeaderItem class
			Infragistics.WebUI.UltraWebGrid.HeaderItem headerItem = (Infragistics.WebUI.UltraWebGrid.HeaderItem) container;

			// Create a new HTML Select element - this is the DropDownList
			System.Web.UI.HtmlControls.HtmlSelect filterDD = new HtmlSelect();

			// Set a unique key for the DropDown if we need to use it later on or for any Client side purpose
			filterDD.ID = "filterDD_" + headerItem.Column.Key;

			// Add a handler for the client-side "onchange" event.  This event is the key event for performing filtering
			// as it will trigger a custom AJAX call to the server to update the grid control.  See the aspx page for the
			// client-side event's implementation.
			filterDD.Attributes.Add("onchange","selectionChange(this)");
			
			// Use the DataSetHelper class to get the distinct values for this column
			ddldsHelper.SelectDistinct(headerItem.Column.Key, gridData.Tables["Orders"], headerItem.Column.Key);

			// Set the DropDownList's DataSource to the table of distinct values
			filterDD.DataSource = gridData.Tables[headerItem.Column.Key];

			// Set the DataTextField of the DropDownList
			filterDD.DataTextField = headerItem.Column.Key;

			// DataBind the DropDownList so that it is populated with the values that we need
			filterDD.DataBind();
			
			// Since we do not need the newly created table for the values in the DropDownList anymore, remove it from the DataSet
			gridData.Tables.Remove(headerItem.Column.Key);

			// You can add any control that you want to actually show the header text for the column.  This example uses a
			// LiteralControl that will be added to the header template to show the column's caption text.
            System.Web.UI.LiteralControl lc = new LiteralControl("<b>" + headerItem.Column.Header.Caption + "</b></br></br>");

			// Add the LiteralControl and the DropDownList to the header template's Controls collection
			container.Controls.Add(lc);
			container.Controls.Add(filterDD);
		}
		#endregion
	}
	#endregion

}
